#ifndef BEACON_ELOG_H_
#define BEACON_ELOG_H_

#include <stdlib.h>
#include <stdio.h>

#include "types.h"

#define B_FATAL      1
#define B_ALARM      2
#define B_ERROR      3
#define B_WARN       4
#define B_INFO       5
#define B_DEBUG      6

#define uint32  unsigned int

extern BOOL g_enable_log;
extern uint32 g_log_level;
extern BOOL g_enable_realtime_test;

#define ELOG(level, fmt, ...)  do {  \
    if(g_enable_log == TRUE && level <= g_log_level ) {  \
        printf("[%s](%s):", __FILE__, __FUNCTION__); \
        printf(fmt, ##__VA_ARGS__); \
		printf("\n"); \
    }  \
} while(0)

#define ELOG_DEBUG(fmt, ...) ELOG(B_DEBUG, fmt, ##__VA_ARGS__)

#define ELOG_INFO(fmt, ...) \
	ELOG(B_INFO, fmt, ##__VA_ARGS__)

#define ELOG_WARN(fmt, ...) \
	ELOG(B_WARN, fmt, ##__VA_ARGS__)

#define ELOG_ERROR(fmt, ...) \
	ELOG(B_ERROR, fmt, ##__VA_ARGS__)

#define ELOG_ALARM(fmt, ...) \
	ELOG(B_ALARM, fmt, ##__VA_ARGS__)

#define ELOG_FATAL(fmt, ...) \
	ELOG(B_FATAL, fmt, ##__VA_ARGS__)

#define jump_if(A) if (A) { goto error; }
#define jump_unless(A) if (!(A)) { goto error; }
#define error_unless(A, M, ...) if (!(A)) { printf(M "\n", ##__VA_ARGS__); goto error; }

#endif